@@ -2,17 +2,23 @@ package autobox::Core;
 
 # XXX ARRAY and HASH should AUTOLOAD, and that AUTOLOAD should look for an array
 # or hash element (double checking with exists) of that method name. This would
-# allow for xmath's ` style dereferences, or javascript's arr.0 and hash.foo 
+# allow for xmath's ` style dereferences, or javascript's arr.0 and hash.foo
 # style dereferences. in perl, you could say $hashref->foo and $arrayref->5.
 # okey, $arrayref->5 is invalid syntax. you'd have to say $five = 5; $arrayref->$five.
 # hrm.
 
+# XXX support 'my IO::Handle $io; $io->open('<', $fn);'. undef values belonging to
+# SVs having associated types should dispatch to that class. of course, just using
+# core, this could be made to work too -- open() is a built-in, after all. the
+# autobox::Core::open would have to know how to handle $_[0] being undef and 
+# assigning the open'ed handle into $_[0].
+
 use 5.8.0;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.3';
+our $VERSION = '0.4';
 
 =pod
 
@@ -31,10 +37,10 @@ autobox::Core - Perl built-in functions exposed as methods in primitive types
 
 Methods wrapping F<perl>'s built-in functions for minipulating numbers, strings, arrays,
 hashes, and code references. It is handy to use built-in functions as methods to avoid
-messy dereferencing syntaxes and parenthesis pile ups. 
+messy dereferencing syntaxes and parenthesis pile ups.
 
 L<autobox> lets you call methods in scalars that aren't object references.
-Numbers, strings, scalars containing numbers, scalars containing strings, 
+Numbers, strings, scalars containing numbers, scalars containing strings,
 array references, hash references, and code references all work as objects.
 L<autobox> adds this feature to L<perl> but does not itself provide any
 methods to call. That is left to the user or another module. For example,
@@ -43,7 +49,7 @@ this module.
 F<autobox::Core> is what you'd call a I<stub> module. It is merely glue, presenting
 existing functions with a new interface. Most of the methods read like
 C<< sub hex ($) { hex($_[0]) } >>.
-Besides built-ins that operate on hashes, arrays, scalars, and code references, 
+Besides built-ins that operate on hashes, arrays, scalars, and code references,
 some Perl 6-ish things were thrown in, and some keyword like C<foreach> have
 been turned into methods.
 
@@ -52,14 +58,15 @@ been turned into methods.
 
 All of the functions listed in L<perldoc> under the headings:
 "Functions for real @ARRAYs",
-"Functions for real %HASHes", 
+"Functions for real %HASHes",
 "Functions for list data",
 and "Functions for SCALARs or strings", plus a few taken from other sections
-and documented below. 
+and documented below.
 Some things expected in Perl 6, such as C<last>, C<elems>, and C<curry>, have been thrown in.
 For use in conjuction with L<Perl6::Contexts>, C<flatten> explicitly flattens an array.
+Functions have been defined for numeric operations.
 
-Of the built-in stuff, the things you use most often on data are all implemented. 
+Of the built-in stuff, the things you use most often on data are all implemented.
 Here's a small sample:
 
   print [10, 20, 30, 40, 50]->pop(), "\n";
@@ -67,7 +74,7 @@ Here's a small sample:
 
   my $arrref = [10, 20, 30];
 
-  my $lala; 
+  my $lala;
   $lala = "Lalalalala\n"; print "chomp: ", $lala->chomp(), ' ', $lala, "\n";
   $lala = "Lalalalala\n"; print "lcfirst: ", $lala->lcfirst(), ' ', $lala, "\n";
 
@@ -80,7 +87,7 @@ C<tied>,
 C<ref>,
 C<undef>,
 C<bless>,
-and C<vec>, where they make sense. 
+and C<vec>, where they make sense.
 C<tie>, C<tied>, and C<undef> don't work on code references, and C<bless> doesn't work on non-reference
 scalars.
 C<quotemeta> works on non-reference scalars, along with C<split>, C<m>, and C<s> for regular expression operations.
@@ -91,7 +98,7 @@ C<quotemeta> works on non-reference scalars, along with C<split>, C<m>, and C<s>
 Array references can tell you how many elements they contain and the index of their last element:
 
   my $arr = [ 1 .. 10 ];
-  print '$arr contains ', $arr->size, 
+  print '$arr contains ', $arr->size,
         ' elements, the last having an index of ', $arr->last, "\n";
 
 Array references have a C<flatten> method to dump their elements.
@@ -110,7 +117,7 @@ Under L<Perl6::Contexts>, you'll often need to write code equivalent to the foll
   do_something(@arr->flatten);
 
 Array references can be iterated on using C<for> and C<foreach>. Both take a code
-reference as the body of the for statement. 
+reference as the body of the for statement.
 C<foreach> passes the current element itself in each pass.
 C<for> passes the index of the current element in to that code block, and then
 the current element, and then a reference to the array itself.
@@ -162,11 +169,16 @@ You may C<curry> code references:
 
   $adding_five_to_numbers->(20)->print; "\n"->print;
 
+These work on numbers:
+
+C<add>, C<and>, C<band>, C<bor>, C<bxor>, C<cmp>, C<dec>, C<div>, C<eq>, C<flip>, C<ge>, C<gt>, C<inc>, C<le>, C<lshift>, C<lt>, C<mod>, C<mult>, C<mcmp>, C<ne>, C<neg>, C<meq>, C<mge>, C<mgt>, C<mle>, C<mlt>, C<mne>, C<not>, C<or>, C<pow>, C<rpt>, C<rshift>, C<sub>, C<xor>.
+
 That's it.
 
 =head2 What's Missing?
 
-Operators. I'm tired. I'll do it in the morning. Maybe. Send me a patch.
+Many operators.  I'm tired.  I'll do it in the morning.  Maybe.  Send me a patch.
+Update:  Someone sent me a patch for numeric operations.
 
 File and socket operations are already implemented in an object-oriented fashion
 care of L<IO::Handle> and L<IO::Socket::INET>.
@@ -179,14 +191,15 @@ Functions listed in the L<perlfunc> headings "System V interprocess communicatio
 "Time-related functions",
 "Keywords related to the control flow of your perl program",
 "Functions for filehandles, files, or directories",
-and 
+and
 "Input and output functions".
 These things are likely implemented in an object oriented fashion by other CPAN
 modules, are keywords and not functions,
 take no arguments,
 or don't make sense as part of the string, number, array, hash, or code API.
-C<srand> because you probably shouldn't be using it. 
-C<each> on hashes. There is no good reason it is missing.
+C<srand> because you probably shouldn't be using it.
+C<each> on hashes. 
+There is no good reason it is missing.
 
 
 =head2 Autoboxing
@@ -232,14 +245,14 @@ All of this applies to hashes and arrays, as well:
   print [ 1 .. 20 ].elems, "\n";
 
 The language is free to implement data storage however it wishes but the programmer
-sees the variables as objects. 
+sees the variables as objects.
 
-Expressions using autoboxing read somewhat like Latin suffixes. 
+Expressions using autoboxing read somewhat like Latin suffixes.
 In the autoboxing mind-set, you might not say that something is "made more mnemonic",
 but has been "mnemonicified".
 
-Autoboxing may be mixed with normal function calls. 
-In the case where the methods are available as functions and the functions are 
+Autoboxing may be mixed with normal function calls.
+In the case where the methods are available as functions and the functions are
 available as methods, it is only a matter of personal taste how the expression should be written:
 
   # Calling methods on numbers and strings, these three lines are equivalent
@@ -249,7 +262,7 @@ available as methods, it is only a matter of personal taste how the expression s
   print 4.sqrt;
   4.sqrt.print;
 
-The first of these three equivalents assumes that a global C<sqrt()> function exists. 
+The first of these three equivalents assumes that a global C<sqrt()> function exists.
 This first example would fail to operate if this global function were removed and only
 a method in the C<Num> package was left.
 
@@ -259,10 +272,10 @@ Perl 5 had the beginnings of autoboxing with filehandles:
   open my $file, '<', 'file.txt' or die $!;
   $file->read(my $data, -s $file);
 
-Here, C<read> is a method on a filehandle we opened but I<never blessed>. 
+Here, C<read> is a method on a filehandle we opened but I<never blessed>.
 This lets us say things like C<< $file->print(...) >> rather than the often ambagious
 
-C<< print $file ... >>. 
+C<< print $file ... >>.
 To many people, much of the time, it makes more conceptual sense as well.
 
 
@@ -273,7 +286,7 @@ What good is all of this?
 =over 1
 
 =item Makes conceptual sense to programmers used to object interfaces as I<the> way
-to perform options. 
+to perform options.
 
 =item Alternative idiom. Doesn't require the programmer
 to write or read expressions with complex precedence rules or strange operators.
@@ -283,14 +296,14 @@ may be rewritten such that the parenthesis span only a few primitive types.
 
 =item Code may often be written with fewer temporary variables.
 
-=item Autoboxing provides the benefits of boxed types without the memory bloat of 
+=item Autoboxing provides the benefits of boxed types without the memory bloat of
 actually using objects to represent primitives. Autoboxing "fakes it".
 
 =item Strings, numbers, arrays, hashes, and so on, each have their own API.
 Documentation for an C<exists> method for arrays doesn't have to explain how hashes are
 handled and vice versa.
- 
-=item Perl tries to accommodate the notion that the "subject" of a statement 
+
+=item Perl tries to accommodate the notion that the "subject" of a statement
 should be the first thing on the line, and autoboxing furthers this agenda.
 
 =back
@@ -300,23 +313,23 @@ Perl is an idiomatic language and this is an important idiom.
 =head3 Subject First: An Aside
 
 Perl's design philosophy promotes the idea that the language should be flexible enough
-to allow programmers to place the X<subject> of a statement first. 
+to allow programmers to place the X<subject> of a statement first.
 For example, C<< die $! unless read $file, 60 >> looks like the primary purpose of the statement is
-to C<die>. 
+to C<die>.
 While that might be the programmers primary goal, when it isn't, the programmer
-can communicate his real primary intention to programmers by reversing the order of 
+can communicate his real primary intention to programmers by reversing the order of
 clauses while keeping the exact same logic: C<< read $file, 60 or die $! >>.
-Autoboxing is another way of putting the subject first. 
+Autoboxing is another way of putting the subject first.
 Nouns make good subjects, and in programming, variables, constants, and object names are the nouns.
-Function and method names are verbs. 
-C<< $noun->verb() >> focuses the readers attention on the thing being acted on rather than the action being performed. 
+Function and method names are verbs.
+C<< $noun->verb() >> focuses the readers attention on the thing being acted on rather than the action being performed.
 Compare to C<< $verb($noun) >>.
 
 
 =head3 Autoboxing and Method Results
 
 In Chapter 11 [Subroutines], we had examples of ways an expression could be
-written. 
+written.
 Here it is again:
 
   # Various ways to do the same thing:
@@ -334,7 +347,7 @@ Here it is again:
   %hash->keys->sort->reverse->print;          # Perl 5 - autobox
 
 This section deals with the last two of these equivalents.
-These are method calls 
+These are method calls
   use autobox;
   use autobox::Core;
   use Perl6::Contexts;
@@ -376,7 +389,7 @@ I must decide how to indent it if it grows to 3 or more lines.
 
 The autoboxed version isn't shorter, but it reads from left to right, and
 the parenthesis from the C<join()> don't span nearly as many characters.
-The complex expression serving as the value being C<join()>ed in the non-autoboxed version 
+The complex expression serving as the value being C<join()>ed in the non-autoboxed version
 becomes, in the autoboxed version, a value to call the C<join()> method on.
 
 This C<print> statement takes a list of CGI parameter names, reads the values for
@@ -384,9 +397,9 @@ each parameter, joins them together with newlines, and prints them with a newlin
 after the last one.
 
 Pretending that this expression were much larger and it had to be broken to span
-several lines, or pretending that comments are to be placed after each part of 
+several lines, or pretending that comments are to be placed after each part of
 the expression, you might reformat it as such:
- 
+
   @cgi_vars->map(sub { CGI::param($_[0]) })  # turn CGI arg names into values
            ->join("\n")                      # join with newlines
            ->concat("\n")                    # give it a trailing newline
@@ -399,7 +412,7 @@ This could also have been written:
            ->concat("\n")                    # give it a trailing newline
            ->print;                          # print them all out
 
-C<map()> is X<polymorphic>. 
+C<map()> is X<polymorphic>.
 The C<map()> method defined in the C<CODE> package takes for its arguments the things
 to map.
 The C<map()> method defined in the C<ARRAY> package takes for its argument a code reference
@@ -410,9 +423,8 @@ I<Here ends the text quoted from the Perl 6 Now manuscript.>
 
 =head1 BUGS
 
-Yes. Report them to the author, L<scott@slowass.net>.
-This code is not well tested. 
-The API is not yet stable - Perl 6-ish things and local extensions are still being renamed.
+Yes. R eport them to the author, L<scott@slowass.net>.
+The API is not yet stable -- Perl 6-ish things and local extensions are still being renamed.
 
 
 =head1 HISTORY
@@ -429,7 +441,7 @@ Version 0.1 was woefully incomplete.
 
 =head1 SEE ALSO
 
-L<autobox>. 
+L<autobox>.
 
 L<Perl6::Contexts>.
 
@@ -438,14 +450,15 @@ Perl 6: L<< http://dev.perl.org/perl6/apocalypse/ >>.
 (Shameless plug alert!) I<Perl 6 Now: The Core Ideas Illustrated with Perl 5>
 dedicates a sizable portion of Chapter 14, Objects, to autoboxing
 and the idea is used heavily throughout the book. Chapter 8, Data Structures,
-also has numerous examples. 
+also has numerous examples.
 See L<http://perl6now.com> or look for ISBN 1-59059-395-2 at your favorite
 bookstore for more information.
 
 
 =head1 AUTHOR
 
-Scott Walters, L<scott@slowass.net>
+Scott Walters, L<scott@slowass.net>.
+Thanks to Matt Spear, who contributed tests and definitions for numeric operations.
 Thanks to chocolateboy for L<autobox> and for the encouragement!
 
 =cut
@@ -485,9 +498,14 @@ sub quotemeta ($) { CORE::quotemeta($_[0]); }
 sub vec     ($$$) { CORE::vec($_[0], $_[1], $_[2]); }
 sub undef   ($)   { $_[0] = undef }
 sub m       ($$)  { [ $_[0] =~ m{$_[1]} ] }
+sub nm       ($$)  { [ $_[0] !~ m{$_[1]} ] }
 sub s       ($$$) { $_[0] =~ s{$_[1]}{$_[2]} }
 sub split   ($$)  { [ split $_[1], $_[0] ] }
 
+sub eval    ($)   { CORE::eval "$_[0]"; }
+sub system  ($;@) { CORE::system @_; }
+sub backtick($)   { `$_[0]`; }
+
 #       Numeric functions
 #           "abs", "atan2", "cos", "exp", "hex", "int", "log",
 #           "oct", "rand", "sin", "sqrt", "srand"
@@ -504,14 +522,59 @@ sub rand    ($)  { CORE::rand($_[0]) }
 sub sin     ($)  { CORE::sin($_[0]) }
 sub sqrt    ($)  { CORE::sqrt($_[0]) }
 
+# functions for array creation
+sub to ($$) { $_[0] < $_[1] ? [$_[0]..$_[1]] : [CORE::reverse $_[1]..$_[0]]}
+sub upto ($$) { [ $_[0]..$_[1] ] }
+sub downto ($$) { [ CORE::reverse $_[1]..$_[0] ] }
+
+# just weird, but cool
+sub times ($&) { for (0..$_[0]-1) { $_[1]->($_); }; $_[0]; }
+
 # doesn't minipulate scalars but works on scalars
 
 sub print   ($;@) { CORE::print @_; }
+sub say     ($;@) { CORE::print @_, "\n"}
 
 # operators that work on scalars:
 
 sub concat ($;@)   { CORE::join '', @_; }
 
+# operator schizzle
+sub add($$) { $_[0] + $_[1]; }
+sub and($$) { $_[0] && $_[1]; }
+sub band($$) { $_[0] & $_[1]; }
+sub bor($$) { $_[0] | $_[1]; }
+sub bxor($$) { $_[0] ^ $_[1]; }
+sub cmp($$) { $_[0] cmp $_[1]; }
+sub dec($) { my $t = CORE::shift @_; --$t; }
+sub div($$) { $_[0] / $_[1]; }
+sub eq($$) { $_[0] eq $_[1]; }
+sub flip($) { ~$_[0]; }
+sub ge($$) { $_[0] ge $_[1]; }
+sub gt($$) { $_[0] gt $_[1]; }
+sub inc($) { my $t = CORE::shift @_; ++$t; }
+sub le($$) { $_[0] le $_[1]; }
+sub lshift($$) { $_[0] << $_[1]; }
+sub lt($$) { $_[0] lt $_[1]; }
+sub mod($$) { $_[0] % $_[1]; }
+sub mult($$) { $_[0] * $_[1]; }
+sub mcmp($$) { $_[0] <=> $_[1]; }
+sub ne($$) { $_[0] ne $_[1]; }
+sub neg($) { -$_[0]; }
+sub meq($$) { $_[0] == $_[1]; }
+sub mge($$) { $_[0] >= $_[1]; }
+sub mgt($$) { $_[0] > $_[1]; }
+sub mle($$) { $_[0] <= $_[1]; }
+sub mlt($$) { $_[0] < $_[1]; }
+sub mne($$) { $_[0] != $_[1]; }
+sub not($) { !$_[0]; }
+sub or($$) { $_[0] || $_[1]; }
+sub pow($$) { $_[0] ** $_[1]; }
+sub rpt($$) { $_[0] x $_[1]; }
+sub rshift($$) { $_[0] >> $_[1]; }
+sub sub($$) { $_[0] - $_[1]; }
+sub xor($$) { $_[0] ^ $_[1]; }
+
 #
 # HASH
 #
@@ -526,6 +589,11 @@ sub exists (\%$) { my $hash = CORE::shift; CORE::exists $hash->{$_[0]}; }
 sub keys (\%) { [ CORE::keys %{$_[0]} ] }
 sub values (\%) { [ CORE::values %{$_[0]} ] }
 
+sub at (\%@) { $_[0]->{@_[1..$#_]}; }
+sub get(\%@) { $_[0]->{@_[1..$#_]}; }
+sub put(\%%) { my $h = CORE::shift @_; my %h = @_; while(my ($k, $v) = CORE::each %h) { $h->{$k} = $v; }; $h; }
+sub set(\%%) { my $h = CORE::shift @_; my %h = @_; while(my ($k, $v) = CORE::each %h) { $h->{$k} = $v; }; $h; }
+
 # local
 
 sub each (\%$) {
@@ -556,7 +624,7 @@ sub lock_keys (\%) { Hash::Util::lock_keys(%{$_[0]}); $_[0]; }
 #
 # ARRAY
 #
-
+##############################################################################################
 package ARRAY;
 
 #       Functions for list data
@@ -567,11 +635,35 @@ sub grep (\@&) { my $arr = CORE::shift; my $sub = CORE::shift; [ CORE::grep { $s
 sub join (\@$) { my $arr = CORE::shift; my $sep = CORE::shift; CORE::join $sep, @$arr; }
 sub map (\@&) { my $arr = CORE::shift; my $sub = shift; [ CORE::map { $sub->($_) } @$arr ]; }
 sub reverse (\@) { [ CORE::reverse @{$_[0]} ] }
-sub sort (\@;&) { my $arr = CORE::shift; my $sub = CORE::shift() || sub { $a cmp $b }; [ CORE::sort { $sub->($a, $b) } @$arr ]; } 
+sub sort (\@;&) { my $arr = CORE::shift; my $sub = CORE::shift() || sub { $a cmp $b }; [ CORE::sort { $sub->($a, $b) } @$arr ]; }
 
 # functionalish stuff
 
-sub sum (\@) { my $arr = CORE::shift; my $res = 0; while(@$arr) { $res += shift @$arr; }; $res; }
+sub sum (\@) { my $arr = CORE::shift; my $res = 0; $res += $_ foreach(@$arr); $res; }
+sub mean(\@) { my $arr = CORE::shift; my $res = 0; $res += $_ foreach(@$arr); $res/@$arr; }
+sub var(\@)
+{
+	my $arr = CORE::shift;
+	my $mean = 0;
+	$mean += $_ foreach(@$arr);
+	$mean /= @$arr;
+	my $res = 0;
+	$res += ($_-$mean)**2 foreach (@$arr);
+	$res/@$arr;
+}
+sub svar(\@)
+{
+	my $arr = CORE::shift;
+	my $mean = 0;
+	$mean += $_ foreach(@$arr);
+	$mean /= @$arr;
+	my $res = 0;
+	$res += ($_-$mean)**2 foreach (@$arr);
+	$res/(@$arr-1);
+}
+
+sub max(\@) { my $arr = CORE::shift; my $max = $arr->[0]; foreach (@$arr) {$max = $_ if $_ > $max }; $max; }
+sub min(\@) { my $arr = CORE::shift; my $min = $arr->[0]; foreach (@$arr) {$min = $_ if $_ < $min }; $min; }
 
 #       Functions for real @ARRAYs
 #           "pop", "push", "shift", "splice", "unshift"
@@ -579,12 +671,18 @@ sub sum (\@) { my $arr = CORE::shift; my $res = 0; while(@$arr) { $res += shift
 sub pop (\@) { CORE::pop @{$_[0]}; }
 sub push (\@;@) { my $arr = CORE::shift; CORE::push @$arr, @_;  $arr; }
 sub unshift (\@;@) { CORE::unshift @{$_[0]}, @_; $_[0]; }
-sub exists (\@$) { my $arr = CORE::shift; CORE::exists $arr->[$_[0]] }
 sub delete (\@$) { my $arr = CORE::shift; CORE::delete $arr->[$_[0]] }
+sub vdelete(\@$) { my $arr = CORE::shift; @$arr = CORE::grep {$_ ne $_[0]} @$arr; }
 sub shift (\@;@) { my $arr = CORE::shift; CORE::shift @$arr; } # last to prevent having to prefix normal shift calls with CORE::
 
 sub undef   ($)   { $_[0] = [] }
 
+# doesn't modify array
+sub exists (\@$) { my $arr = CORE::shift; CORE::scalar(CORE::grep {$_ eq $_[0]} @$arr) > 0; }
+sub at(\@$) { my $arr = CORE::shift; $arr->[$_[0]]; }
+sub count(\@$) { my $arr = CORE::shift; scalar(CORE::grep {$_ eq $_[0]} @$arr); }
+sub uniq(\@) { my $arr = CORE::shift; my %h; [ CORE::map { $h{$_}++ == 0 ? $_ : () } @$arr ] } # shamelessly from List::MoreUtils
+
 # tied and blessed
 
 sub bless (\@$)   { CORE::bless $_[0], $_[1] }
@@ -594,6 +692,7 @@ sub ref   (\@)    { CORE::ref   $_[0] }
 
 # perl 6-ish extensions to Perl 5 core stuff
 
+sub first(\@) { my $arr = CORE::shift; $arr->[0]; }
 sub last (\@) { my $arr = CORE::shift; $#$arr; }
 sub size (\@) { my $arr = CORE::shift; CORE::scalar @$arr; }
 sub elems (\@) { my $arr = CORE::shift; CORE::scalar @$arr; } # Larry announced it would be elems, not size
@@ -601,35 +700,38 @@ sub length (\@) { my $arr = CORE::shift; CORE::scalar @$arr; }
 
 # misc
 
-sub each (\@$) { 
+sub each (\@$) {
     # same as foreach(), apo12 mentions this
-    my $arr = CORE::shift; my $sub = CORE::shift; 
+    my $arr = CORE::shift; my $sub = CORE::shift;
     foreach my $i (@$arr) {
         $sub->($i);
     }
 }
 
-sub foreach (\@$) { 
-    my $arr = CORE::shift; my $sub = CORE::shift; 
+sub foreach (\@$) {
+    my $arr = CORE::shift; my $sub = CORE::shift;
     foreach my $i (@$arr) {
         $sub->($i);
     }
 }
 
-sub for (\@$) { 
-    my $arr = CORE::shift; my $sub = CORE::shift; 
+sub for (\@$) {
+    my $arr = CORE::shift; my $sub = CORE::shift;
     for(my $i = 0; $i < $#$arr; $i++) {
         $sub->($i, $arr->[$i], $arr);
     }
 }
 
 sub print   (\@) { my $arr = CORE::shift; my @arr = @$arr; CORE::print "@arr"; }
+sub say   (\@) { my $arr = CORE::shift; my @arr = @$arr; CORE::print "@arr\n"; }
 
 # local
 
 sub elements (\@) { ( @{$_[0]} ) }
 sub flatten (\@) { ( @{$_[0]} ) }
 
+##############################################################################################
+
 #
 # CODE
 #
